#include "TM1636.h"
#include <EEPROM.h>

#define TERM      A0
#define PHOTO     A1
#define MENU_KEY  11
#define BUZZER    6
#define D4        5
#define D3        5
#define D2        5
#define D1        5

//EEPROM ADRESY
//offset 0 - CorF (0=F, 1=C)
//offset 1 - maxc
//offset 2 - maxf
//offset 3 - minc
//offset 4 - minf

bool celsius = true;
bool turnOnBuzz = false;
bool buzzerState = false;

unsigned long cas_nyni = 0l;
unsigned long cas_minuly = 0l;

int lcdChars[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
int addresa = 0;

TM1636 LCD(7, 8);

//Zapis do displeje
void lcdWrite(int temperature, bool fahrenheit) {
  int CorF = 12;    //v zakladu C
  
  if (fahrenheit) {
    CorF = 15;   //C nebo F
    temperature = CtoF(temperature);
  }
  
  int8_t data[] = {17, (temperature/10)%10, temperature%10, CorF}; //datova struktua lcd, 17=prazdne policko, temperature/10)%10 - desitky
                                                                   //temperature%10 - jednotky, 12 - 'C', 15 - 'F'  
  LCD.start();
  LCD.display(data);
  LCD.stop();
}

//Prevod teploty
int CtoF(int temperature) {
  return temperature * (5/9) + 32;
}

//Vypocet teploty
int calcTemp() {
  int B = 3975;
  int termVoltage = analogRead(TERM);   //cteni napeti na termistoru
  float termResist = (1023.0f-termVoltage) * 10000.0f/termVoltage;  //vypocet odporu termistoru
  
  return 1/(log(termResist/10000)/B+1/298.15)-273.15;
}

//Funkce piskani
void buzz(float hz) {
  if((cas_nyni - cas_minuly) >= ((1.0f/hz)/2.0f * 1000)) {     //Prevod Hz na pul periodu
    digitalWrite(BUZZER, buzzerState);
    buzzerState = !buzzerState;
    cas_minuly = cas_nyni;
  }
}

//------------------------------------------------------------------------------------------------------------------------

void setup() {
  Serial.begin(9600);
  pinMode(PHOTO, INPUT);
  pinMode(TERM, INPUT);
  pinMode(MENU_KEY, INPUT_PULLUP);

  pinMode(BUZZER, OUTPUT);

  LCD.init();
}

void loop() {
  cas_nyni = millis();

  if(turnOnBuzz) buzz(5);

  if(!digitalRead(MENU_KEY) && (cas_nyni - cas_minuly) >= 200 ) {
    if(celsius == true) celsius = false;
    else celsius = true;

    EEPROM.write(0, celsius);
    cas_minuly = cas_nyni;
  }
  
  if(analogRead(PHOTO) > 400) {
    int actualTemp = calcTemp(); 
    lcdWrite(actualTemp, EEPROM.read(0));

    //Ukladani max teplot
    if(EEPROM.read(1) < actualTemp) {
      EEPROM.write(1, actualTemp);    //Pokud je aktualni teplota vyssi nez maximalni v pameti, prepis
      EEPROM.write(2, CtoF(actualTemp));
    }

    //Ukladani min teplot
    EEPROM.write(3, 255);   //nastaveni 255 urovne teploty v registru minimalnich teplot
    
    if(EEPROM.read(3) > actualTemp) {
      EEPROM.write(3, actualTemp);    //Pokud je aktualni teplota vyssi nez maximalni v pameti, prepis
      EEPROM.write(4, CtoF(actualTemp));
    }  
    
    if((cas_nyni - cas_minuly) >= 30000) {
      turnOnBuzz = true;
    }
    
  } else if(analogRead(PHOTO) < 400) {
      turnOnBuzz = false;
      LCD.clearDisplay();
      cas_minuly = cas_nyni;
  }
}
